/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.ant;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.management.Attribute;
import javax.management.AttributeNotFoundException;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.ReflectionException;

import org.apache.tools.ant.BuildException;

/**
 * This class is used to gathers the operation on the components
 * 
 * @author ddesjardins chamerling - eBMWebsourcing
 */
public abstract class AbstractComponentAntTask extends AbstractJBIAntTask {

    /**
     * The nested elements <param>
     */
    private List<Param> nestedParams = new ArrayList<Param>();

    /**
     * Location of the nestedParams file that contains MBean properties
     */
    private String params;

    /**
     * Action : shutdown the component
     */
    protected static final String SHUTDOWN = "shutDown";

    /**
     * Action : start the component
     */
    protected static final String START = "start";

    /**
     * Action : stop the component
     */
    protected static final String STOP = "stop";

    /**
     * Action : Uninstall the component
     */
    protected static final String UNINSTALL = "uninstall";

    /**
     * Name of the JBI component in the JBI environment
     */
    protected String name;

    /**
     * The action to perform
     */
    private String action;

    /**
     * Creates a new instance of {@link AbstractComponentAntTask}
     * 
     * @param action
     */
    public AbstractComponentAntTask(String action) {
        this.action = action;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.ant.managers.AbstractJBIAntTask#doTask()
     */
    public void doTask() throws Exception {
        if (name == null) {
            throw new BuildException("Missing attribute 'name'");
        }

        if (UNINSTALL.equalsIgnoreCase(action)) {
            performUninstall();
        } else {
            performAction();
        }
    }

    /**
     * Perform an action on the component
     * 
     * @param action
     *            action to perform
     */
    protected void performAction() throws Exception {
        MBeanServerConnection connection = connector.getMBeanServerConnection();

        Object[] parameters = new Object[0];
        String[] signature = new String[0];
        ObjectName objectName = JBIJMXConnectorUtil.getComponentMBeanName(connection, name);

        if (objectName == null) {
            throw new BuildException("The component '" + name + "' is not installed");
        }

        try {
            log("Invoking '" + action + "' on component '" + name + "'");
        } catch (NullPointerException e) {
            // Exception thrown when execute outisde of the ant context
        }

        connection.invoke(objectName, action, parameters, signature);
    }

    /**
     * Uninstall a component
     * 
     * @param connector
     * @throws Exception
     */
    protected void performUninstall() throws BuildException {
        MBeanServerConnection connection = null;
        try {
            connection = connector.getMBeanServerConnection();

            Object[] parameters1 = new Object[1];
            Object[] parameters2 = new Object[0];
            parameters1[0] = name;
            String[] signature1 = new String[] { "java.lang.String" };
            String[] signature2 = new String[] {};
            // First call uninstall on installer
            ObjectName instName = (ObjectName) connection.invoke(JBIJMXConnectorUtil
                    .getInstallationServiceMBeanName(connection), "loadInstaller", parameters1,
                    signature1);

            // Get the installer configuration
            Object result1 = connection.getAttribute(instName, "InstallerConfigurationMBean");
            System.err.println("Getting the InstallerConfigurationMBean:" + result1);
            System.err.println("" + nestedParams.size() + " parameters to be set");
            if (result1 instanceof ObjectName) {
                // set the properties and parameters
                ObjectName configurationMBean = (ObjectName) result1;
                if (nestedParams.size() > 0 || params != null) {
                    setConnectionAttributeFromPropertiesFile(connection, configurationMBean);
                    setConnectionAttributeFromNestedParam(connection, configurationMBean);
                }
            }

            connection.invoke(instName, "uninstall", parameters2, signature2);

        } catch (IOException e) {
            throw new BuildException("The component '" + name + "' has not been uninstalled: "
                    + e.getMessage());
        } catch (InstanceNotFoundException e) {
            throw new BuildException("The component '" + name + "' has not been uninstalled: "
                    + e.getMessage());
        } catch (MalformedObjectNameException e) {
            throw new BuildException("The component '" + name + "' has not been uninstalled: "
                    + e.getMessage());
        } catch (MBeanException e) {
            throw new BuildException("The component '" + name + "' has not been uninstalled: "
                    + e.getMessage());
        } catch (ReflectionException e) {
            throw new BuildException("The component '" + name + "' has not been uninstalled: "
                    + e.getMessage());
        } catch (NullPointerException e) {
            throw new BuildException("The component '" + name + "' has not been uninstalled: "
                    + e.getMessage());
        } catch (AttributeNotFoundException e) {
            throw new BuildException("The component '" + name + "' has not been uninstalled: "
                    + e.getMessage());
        } catch (Exception e) {
            throw new BuildException("The component '" + name + "' has not been uninstalled: "
                    + e.getMessage());
        } finally {
            // Unload the installer
            try {
                if (connection != null) {
                    Object[] parameters3 = new Object[2];
                    parameters3[0] = name;
                    parameters3[1] = true;
                    String[] signature3 = new String[] { "java.lang.String", "boolean" };
                    boolean result;
                    result = ((Boolean) connection.invoke(JBIJMXConnectorUtil
                            .getInstallationServiceMBeanName(connection), "unloadInstaller",
                            parameters3, signature3)).booleanValue();
                    if (!result) {
                        throw new BuildException("The component '" + name
                                + "' has not been uninstalled");
                    }
                    log("The component '" + name + "' has been uninstalled");
                }
            } catch (InstanceNotFoundException e) {
                throw new BuildException("The component '" + name + "' has not been uninstalled: "
                        + e.getMessage());
            } catch (MalformedObjectNameException e) {
                throw new BuildException("The component '" + name + "' has not been uninstalled: "
                        + e.getMessage());
            } catch (MBeanException e) {
                throw new BuildException("The component '" + name + "' has not been uninstalled: "
                        + e.getMessage());
            } catch (ReflectionException e) {
                throw new BuildException("The component '" + name + "' has not been uninstalled: "
                        + e.getMessage());
            } catch (NullPointerException e) {
                throw new BuildException("The component '" + name + "' has not been uninstalled: "
                        + e.getMessage());
            } catch (IOException e) {
                throw new BuildException("The component '" + name + "' has not been uninstalled: "
                        + e.getMessage());
            }

        }
    }

    /**
     * 
     * @param name
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * Get the properties from the params file.
     * 
     * @return
     * @throws IOException
     */
    private Properties loadProperties() throws IOException {
        Properties props = new Properties();
        if (params != null)
            props.load(new FileInputStream(params));

        return props;
    }

    /**
     * Set the connection attributes from the properties file
     * 
     * @nestedParams connection
     * @nestedParams configurationMBean
     * @throws Exception
     */
    private void setConnectionAttributeFromPropertiesFile(MBeanServerConnection connection,
            ObjectName configurationMBean) throws Exception {
        Properties props = loadProperties();
        for (Map.Entry<Object, Object> entry : props.entrySet()) {
            Attribute attribute = new Attribute((String) entry.getKey(), (String) entry.getValue());
            connection.setAttribute(configurationMBean, attribute);
        }
    }

    /**
     * Set the connection attribute from the nested params
     * 
     * @nestedParams connection
     * @nestedParams configurationMBean
     * @throws Exception
     */
    private void setConnectionAttributeFromNestedParam(MBeanServerConnection connection,
            ObjectName configurationMBean) throws Exception {
        // Set the properties of the <Param> element
        for (Param p : nestedParams) {
            System.err.println("Setting attribute " + p.getName() + " = " + p.getValue());
            Attribute attribute = new Attribute(p.getName(), p.getValue());
            connection.setAttribute(configurationMBean, attribute);
        }
    }

    /**
     * @return the params
     */
    public String getParams() {
        return params;
    }

    /**
     * @nestedParams params the params to set
     */
    public void setParams(String params) {
        this.params = params;
    }

    /**
     * 
     * @return
     */
    public Param createParam() {
        Param param = new Param();
        nestedParams.add(param);
        return param;
    }

    /**
     * Parameter class for nested <param> elements
     * 
     * @author chamerling
     * 
     */
    public static class Param {
        /**
         * Name attribute
         */
        private String name;

        /**
         * Value attribute
         */
        private String value;

        /**
         * Creates a new instance of {@link Param}
         * 
         */
        public Param() {
            this("", "");
        }

        /**
         * Creates a new instance of {@link Param}
         * 
         * @param name
         * @param value
         */
        public Param(String name, String value) {
            this.name = name;
            this.value = value;
        }

        /**
         * @return the name
         */
        public String getName() {
            return name;
        }

        /**
         * @param name
         *            the name to set
         */
        public void setName(String name) {
            this.name = name;
        }

        /**
         * @return the value
         */
        public String getValue() {
            return value;
        }

        /**
         * @param value
         *            the value to set
         */
        public void setValue(String value) {
            this.value = value;
        }

    }

}
